import { debug } from 'console';
import express, {NextFunction, Request,Response} from 'express';
import mysql from 'mysql2/promise';
import cors from 'cors';
import http, { createServer } from 'http';
import { Server } from 'socket.io';

let app=express();
const server = createServer(app);

const io = new Server(server, {
    cors:{
        origin: "http://localhost:3000",
        methods:["GET", "POST"]
    }
});
let nbClients:number=0;
io.on("connection", (socket) => {
    nbClients++;
    console.log("Un utilisateur connecté : ", nbClients);
    io.emit("nbClients", nbClients); 

    socket.on("disconnect", () => {
        nbClients--;
        console.log("Utilisateur déconnecté :", nbClients);
        io.emit("nbClients", nbClients); 
    }); 
});

interface User{
    id?:number;
    nom:string;
    email:string;
    password:string;
}
let users:User[]=[];
let tokens:string[]=[];

const db=mysql.createPool({
    host: "localhost",
    user: "IIR2024S1",
    password: "IIR2024S1",
    database: "bltname_IIR2024S1" });

app.use(express.json());
app.use(cors());
app.use((req:Request, res:Response, next:NextFunction)=>{
    console.log(`[${new Date().toISOString()}] ${req.method} ${req.path}`);
    if(req.path.startsWith("/api/users")){
        if(tokens.includes(req.headers.authorization?.split(" ").pop()??"")) next();
        else
            res.status(403).json({error:"Accès non autorisé!"});
    }else
        next();
})

app.get('/',(req:Request, res:Response)=>{
    res.send("Bienvenue sur votre serveur Node.js avec TypeScript !");
});


app.get('/api/users',async(req:Request, res:Response)=>{
    try{
    let [results]= await db.query("SELECT id, nom, email, password FROM users");
    res.json(results);
    }catch(err){
        res.status(500).json({erro:"Erreur lors du listing",debug:err});
    }
});
app.post('/api/users',async (req:Request, res:Response)=>{
    const user=req.body;
    if(user.nom){
        users.push(user);
        try{
        let [result]= await db.query("INSERT into users (nom, email, password) VALUES (?,?,MD5(?))",
            [user.nom, user.email??"", user.password??""])
        res.json({success:"Utilisateur ajouté avec succès",debug:result});
        }catch(err){
            res.json({erro:"Erreur lors de l'insertion",debug:err});
        }
    }else
        res.status(400).json({error:"Ce n'est pas un objet User valide!"});
});
app.get('/api/users/:id',async (req:Request, res:Response)=>{
    const id=parseInt(req.params.id);
    try{
    let [results]= await db.query("SELECT id, nom, email, password FROM users WHERE id=?",[id]);
    if(Array.isArray(results) && (results.length==0)) res.status(404).json({error:"Utilisateur non trouvé!"});
    else res.json(results);
    }catch(err){
        res.status(500).json({error:"Erreur lors de la recherche",debug:err});
    }
    // let user=users.find((u)=>u.id===id);
    // if(user)res.json(user);
    // else res.status(404).json({error:"Utilisateur non existant!"});
});
app.put('/api/users/:id',async (req:Request, res:Response)=>{
    const id=parseInt(req.params.id);
    const {nom, email, password}=req.body;
    ///TODO: valider user
    try{
    let [results]= await db.query("UPDATE users SET nom=?, email=?, password=MD5(?) WHERE id=?",[nom,email,password,id]);
    res.json({success:"Utilisateur modifié avec succès"});
    }catch(err){
        res.status(500).json({error:"Erreur lors de la modification",debug:err});
    }
});
app.delete('/api/users/:id',async (req:Request, res:Response)=>{
    const id=parseInt(req.params.id);
    try{
        let [results]= await db.query("DELETE FROM users WHERE id=?",[id]);
        res.json({success:"Utilisateur supprimé avec succès"});
    }catch(err){
        res.status(500).json({error:"Erreur lors de la suppression",debug:err});
    }
});
app.post('/api/login',async (req:Request, res:Response)=>{
    const {email, password}=req.body;
    try{
        let [users]= await db.query("SELECT id FROM users WHERE email=? AND password=MD5(?)",[email, password]);
        if(Array.isArray(users) && users.length){
            const token=Math.random().toString(24).substring(0,10);
            tokens.push(token);
            res.json({token:token});
        }else
            res.status(404).json({error:"Utilisateur non trouvé!"});
    }catch(err){
        res.status(500).json({error:"Erreur lors de l'authentification",debug:err});
    }
});

app.use((req:Request, res:Response, next:NextFunction)=>{
    if(req.path.startsWith("/api"))
        res.status(404).json({error:"API non existante!"})
    else
        res.status(404).send("<h1>Page non existante! 404</h1>")
})

const PORT=3001;

server.listen(PORT, ()=>{
    console.log(`Application démarrée sur ${PORT}`);
})